#include "stm32l_discovery_lcd.h"

//Объявляем структуры
GPIO_InitTypeDef    GPIO_InitStructure;
LCD_InitTypeDef     LCD_InitStruct;
ADC_InitTypeDef     ADC_InitStruct;

uint16_t ADC_Data; //Переменная для хранения результата преобразования ADC
uint16_t strDisp[6]; //Массив символов для вывода на дисплей

//Объявляем функцию, которая конвертирует результат преобразования ADC в массив символов ASCII 
void convert_into_char(uint32_t number, uint16_t *p_tab);

int main()
{
  //Выбор источника тактирования SYSCLK
  RCC_HSICmd(ENABLE); //Включаем внутренний генератор HSI - 16 МГц
  while (RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == RESET); //Ждем стабилизации HSI
  RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI); //Выбираем HSI в качестве источника SYSCLK
  
  //Разрешаем тактирование портов A, B, C  
  RCC_AHBPeriphClockCmd((RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOC), ENABLE);
  
  //Инициализация выводов порта A для работы LCD
  //Задаем в структуре нужные выводы порта для управления LCD
  GPIO_InitStructure.GPIO_Pin = (GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 
                                 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 
                                   | GPIO_Pin_15);
  /*Эти выводы конфигурируем на работу с альтернативной функцией.
  С какой именно задаем ниже*/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  //Инициализируем порт A с помощью созданной структуры
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  /*Отдельно для каждого вывода выбираем альтернативную функцию работы с LCD.
  GPIO_PinSource1..15 можно заменить GPIO_Pin_1..15 - это равнозначно*/  
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource15, GPIO_AF_LCD);
  
  //Инициализация вывода PA5 порта А для работы с ADC
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5; //Вывод порта для работы с ADC
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN; //Аналоговый режим для вывода PA5
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL; //Отключаем подтягивающие резисторы
  GPIO_Init(GPIOA, &GPIO_InitStructure); //Конфигурируем вывод 
  
  /*Инициализация выводов порта B для работы с LCD. Последовательность та же, 
  только выводы другие*/
  GPIO_InitStructure.GPIO_Pin = (GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 
                                 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 
                                   | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 
                                     | GPIO_Pin_14 | GPIO_Pin_15);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource12, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_LCD);
  
  //Инициализация выводов порта C для работы с LCD
  GPIO_InitStructure.GPIO_Pin = (GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 
                                 | GPIO_Pin_3 | GPIO_Pin_6 |GPIO_Pin_7 
                                   | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 
                                     | GPIO_Pin_11);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource0, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource2, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource3, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_LCD);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_LCD);
  
  //Инициализация ADC
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); //Включаем тактирование ADC
  ADC_InitStruct.ADC_Resolution = ADC_Resolution_12b;  //Разрешение ADC - 12 бит
  ADC_InitStruct.ADC_DataAlign = ADC_DataAlign_Right;  //Выравнивание результата вправо
  ADC_Init(ADC1, &ADC_InitStruct);  //Конифигурируем модуль ADC заданной структурой
  /*Далее делаем следующие настройки для регулярного канала: 
  Выбор ADC - ADC1,
  канал № 5,
  число преобразований в последовательности - 1, 
  sample time - 16 тактов*/
  ADC_RegularChannelConfig(ADC1, ADC_Channel_5, 1, ADC_SampleTime_16Cycles);
  ADC_Cmd(ADC1, ENABLE); //И включаем ADC
  while(ADC_GetFlagStatus(ADC1, ADC_FLAG_ADONS) == RESET); //Ждем готовности ADC
  
  //Задаем источник тактирования для LCD
  //Разрешаем тактирование модулей LCD и PWR
  RCC_APB1PeriphClockCmd((RCC_APB1Periph_PWR | RCC_APB1Periph_LCD), ENABLE);
  //Разрешаем доступ к регистрам RTC
  PWR_RTCAccessCmd(ENABLE);
  //Включаем внешний генератор LSE - 32,768 кГц 
  RCC_LSEConfig(RCC_LSE_ON);
  //Ждем его готовности
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET);
  //Включаем тактирование модуля RTC
  RCC_RTCCLKCmd(ENABLE);
  //Выбираем LSE в качестве источника тактирования RTC (соответственно и LCD)
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
    
  //Инициализация LCD
  LCD_InitStruct.LCD_Prescaler = LCD_Prescaler_16; //CLKPS = LCDCLK/16
  LCD_InitStruct.LCD_Divider = LCD_Divider_17; //LCD frequency = CLKPS/17
  LCD_InitStruct.LCD_Duty = LCD_Duty_1_4; //1/4 duty
  LCD_InitStruct.LCD_Bias = LCD_Bias_1_3; //1/3 Bias
  //В качестве источника питания для LCD выбираем внутренний step-up конвертер
  LCD_InitStruct.LCD_VoltageSource = LCD_VoltageSource_Internal; 
  LCD_Init(&LCD_InitStruct); //Конфигурируем LCD
  
  LCD_MuxSegmentCmd(ENABLE); //Делаем "ремаппинг" выводов
  LCD_ContrastConfig(LCD_Contrast_Level_0); //Задана минимальная контрастность
  LCD_WaitForSynchro(); //Ждем синхронизации регистра LCD_FCR
  LCD_Cmd(ENABLE); //разрешаем работу контроллера LCD
  //Ждем готовности step-up конвертера и контроллера LCD
  LCD_GetFlagStatus(LCD_FLAG_RDY | LCD_FLAG_ENS); 
  /*На этом инициализация LCD закончена. Для вывода символов будем использовать
    функцию LCD_GLASS_DisplayStrDeci() из демо-проекта*/
  while(1)
  {
    ADC_SoftwareStartConv(ADC1); //Запускаем преобразование
    ADC_Data = ADC_GetConversionValue(ADC1); //Считываем результат преобразования
    ADC_Data = ADC_Data*3000/4096; //Масштабируем. Величина 3 Волта выбрана приблизительно
    /*Конвертируем результат преобразования ADC в массив символов.
    Результат сохраняется в элементах массива 1..4*/
    convert_into_char(ADC_Data, strDisp); 
    strDisp[0] = ' '; //Первое знакоместо дисплея оставляем пустым
    strDisp[4] = ' '; //Четвертое знакоместо дисплея тоже стираем. Здесь были единицы милливольт
    strDisp[5] = 'V'; //В последнее знакоместо дисплея выводим букву V
    strDisp[1] |= DOT; //Десятичная точка
    /*Следующая функция выводит на экран массив strDisp. Еще она позволяет выводить на 
    экран десятичную точку или двоеточие  в любой позиции экрана. 
    В данном случае команда strDisp[1] |= DOT выводит точку после первой цифры*/ 
    LCD_GLASS_DisplayStrDeci(strDisp);
  }
}

//Эта функция конвертирует результат преобразования ADC в массив символов
void convert_into_char(uint32_t number, uint16_t *p_tab)
{
  uint16_t units=0, tens=0, hundreds=0, thousands=0, misc=0;
  
  units = (((number%10000)%1000)%100)%10;
  tens = ((((number-units)/10)%1000)%100)%10;
  hundreds = (((number-tens-units)/100))%100%10;
  thousands = ((number-hundreds-tens-units)/1000)%10;
  misc = ((number-thousands-hundreds-tens-units)/10000);
  
  *(p_tab+4) = units + 0x30;
  *(p_tab+3) = tens + 0x30;
  *(p_tab+2) = hundreds + 0x30;
  *(p_tab+1) = thousands + 0x30;
  *(p_tab) = misc + 0x30;
}
#ifdef  USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{
  while (1)
  {
  }
}
#endif
